'use strict';

/* Controllers */

angular.module('licenseServerApp.controllers', ['licenseServerApp.services'])
.controller('IndexController', function($scope, $location) {            
      $scope.placeholder = 'Search';
      $scope.isDateSearch = false;
     
      $scope.isActive = function (currentURL) {
          return currentURL === $location.path();
      };    

      //$scope.$watch('isDateSearch', function () {
      //    $scope.placeholder = $scope.isDateSearch ? 'dd/mm/yyyy' : 'Search';
      //});    

      $scope.validateDate = function (searchText) {
          if ($scope.isDateSearch) {
              if (angular.isUndefined(searchText)) {                  
                  return false;
              }
             
              if (parseDate(searchText) == null) {                  
                  return false;
              }
              return true;             
          }
      };

      function parseDate(str) {
          function pad(x){return (((''+x).length==2) ? '' : '0') + x; }
          var m = str.match(/^(\d{1,2})\/(\d{1,2})\/(\d{4})$/)
            , d = (m) ? new Date(m[3], m[2]-1, m[1]) : null
            , matchesPadded = (d&&(str==[pad(d.getDate()),pad(d.getMonth()+1),d.getFullYear()].join('/')))
            , matchesNonPadded = (d&&(str==[d.getDate(),d.getMonth()+1,d.getFullYear()].join('/')));
          return (matchesPadded || matchesNonPadded) ? d : null;
      }

  })  
.controller('ServerInfoController', function ($scope, $http) {       
    $scope.hideReason = true;    
    $scope.refresh = function () {
      
        var futureResponse =
        $http(
                {
                    url: '/api/serverinformation',
                    method: 'GET'
                }
                );
        futureResponse.success(function (data, status, headers, config) {
            $scope.serverInfo = data;            
            $scope.hideReason = $scope.serverInfo.IsValid;
        }
        );
    }
    $scope.refresh();
})
.controller('LicensesController', function ($scope, $http, $modal, ErrorCode) {
    var inverse = '-';
    var empty = '';
    var separator = ',';
    var revokeWarningMessage = 'You are attempting to revoke more than 5 licenses. ' +
                               'You should only be able to unblock X users per day. ' +
                               'Do you want to continue?';
    var userBlocked = 'The user has been blocked.';
    var licenseGranted = 'The license has been granted';
    var blockUserError = '';
    var grantLicenseError = '';
    var updateError = '';
    $scope.UserIDList = empty;
    $scope.blockUserList = empty;
    $scope.licenses = {};    
    $scope.orderItem = 'UserId';
    $scope.selectAll = false;
    $scope.canRefresh = false;
    $scope.nullDate = '0001-01-01T00:00:00';


    $scope.sort = function (sortItem) {
        $scope.orderItem = sortItem;
    }

    $scope.inverseSort = function (sortItem) {
        $scope.orderItem = inverse.concat(sortItem);
    }

    $scope.refresh = function () {
        var futureResponse =
        $http(
                {
                    url: '/api/licenses',
                    method: 'GET'
                }
            );

        futureResponse.success(function (data, status, headers, config) {
            $scope.licenses = data;
            $scope.UserIDList = empty;
            $scope.blockUserList = empty;
            $scope.selectAll = false;
        }
      );
    }    

    $scope.revoke = function () {
        var canRevoke = true;
        var reponseCount = 0;
        var revokeCount = $scope.checkedCount();        
        if (revokeCount > 5) {
            canRevoke = false;
            if (window.confirm(revokeWarningMessage)) {
                canRevoke = true;
            }
        }
        if (canRevoke) {
            $scope.canRefresh = false;
            angular.forEach($scope.licenses, function (license) {

                if (license.checked) {
                    var futureResponse =
                    $http(
                            {
                                url: '/api/licenses/' + license.Id,
                                method: 'DELETE'
                            }
                        );
                    futureResponse.success(function (data, status, headers, config) {
                        reponseCount++;
                        if (reponseCount == revokeCount) {
                            $scope.canRefresh = true;
                            $scope.showDialog(userBlocked, true);
                        }
                    });
                    futureResponse.error(function (data, status, headers, config) {
                        reponseCount++;
                        if (reposneCount == revokeCount) {
                            $scope.canRefresh = true;
                        }
                        if (status == ErrorCode.NotFound) {
                            blockUserError = 'User has been blocked already';
                        } else if (status == ErrorCode.InternalServerError) {
                            blockUserError = data;
                        }
                        else {
                            blockUserError = status;
                        }

                        $scope.showDialog(blockUserError,false);                        
                    });
                }
            });           
        }
    }
    
    $scope.grant = function () {
        $scope.canRefresh = false;
        if ($scope.UserIDList != empty) {
            var userArray = $scope.validateAndListUser($scope.UserIDList);
            var userCount = userArray.length;
            var reponseCount = 0;
            if (userCount > 0) {
                angular.forEach(userArray, function (userId) {
                    var license = {
                        UserId: userId,
                        Id: null,
                        LastSessionStart: null,
                        LastSessionSystem: null,
                    };
                    var futureResponse =
                        $http(
                        {
                            url: '/api/licenses',
                            method: 'POST',
                            data: license
                        }
                    );
                    futureResponse.success(function (data, status, headers, config) {
                        reponseCount++;
                        if (reponseCount == userCount) {
                            $scope.canRefresh = true;
                            $scope.showDialog(licenseGranted,true);
                        }
                    });
                    futureResponse.error(function (data, status, headers, config) {
                        reponseCount++;
                        if (reponseCount == userCount) {
                            $scope.canRefresh = true;
                        }
                        if (status == ErrorCode.Conflict) {
                            grantLicenseError = 'License has been given to this user already';
                        } else if (status == ErrorCode.NotModified) {
                            grantLicenseError = 'No more licenses available';
                        } else if (status == ErrorCode.InternalServerError) {
                            grantLicenseError = data;
                        }
                        else {
                            grantLicenseError = status;
                        }

                        $scope.showDialog(grantLicenseError,false);                        
                    });

                });               
            }
        }      
        
    }        

    $scope.updateLicense = 
        function (license) 
        {
            var futureResponse =
                $http(
                {
                    url: '/api/licenses/' + license.Id,
                    method: 'PUT',
                    data: license
                }
            );
            futureResponse.error(function (data, status, headers, config) {
                config.data.AllowOffline = !config.data.AllowOffline;
                if (status == ErrorCode.NotFound) {
                    updateError = 'User is not available to grant offline permission.';
                } else {
                    updateError = status;
                }
                $scope.showDialog(updateError, false);
            });
        }

    $scope.blockUser = function () {
        var canBlock = false;
        $scope.canRefresh = false;
        if ($scope.blockUserList != empty) {
            var blockUserArray = $scope.validateAndListUser($scope.blockUserList);
            var blockUserCount = blockUserArray.length;
            var reponseCount = 0;
            if (blockUserCount > 0) {
                canBlock = true;
                if (blockUserCount > 5) {
                    canBlock = false;
                    if (window.confirm(warningOnBlock)) {
                        canBlock = true;
                    }
                }
                if (canBlock) {
                    angular.forEach(blockUserArray, function (blockUserId) {
                        var lockedUser = {
                            UserId: blockUserId,
                            Id: null
                        };
                        var futureResponse =
                            $http(
                            {
                                url: '/api/lockedUsers',
                                method: 'POST',
                                data: lockedUser
                            }
                        );
                        futureResponse.success(function (data, status, headers, config) {
                            reponseCount++;
                            if (reponseCount == blockUserCount) {
                                $scope.canRefresh = true;
                                $scope.showDialog(userBlocked,true);
                            }
                        });
                        futureResponse.error(function (data, status, headers, config) {
                            reponseCount++;
                            if (reponseCount == blockUserCount) {
                                $scope.canRefresh = true;
                            }
                            if (status == ErrorCode.Conflict) {
                                blockUserError = 'User has been blocked already';
                            } else if (status == ErrorCode.NotModified) {
                                blockUserError = 'No more user can be blocked for today.';
                            }else if (status == ErrorCode.InternalServerError) {
                                blockUserError = data;
                            }
                            else {
                                blockUserError = status;
                            }
                            $scope.showDialog(blockUserError,false);                            
                        });

                    });
                }
            }
        }
    }

    $scope.checkedCount = function () {
        return $scope.licenses.filter(function (license) {
            return license.checked;
        }).length;
    }

    $scope.validateAndListUser = function (itemArray) {
        var list = itemArray.split(separator);
        var lcount = list.length;
        var user;
        var userList = new Array();
        for (var i = 0; i < lcount; i++) {
            user = list[i].trim();
            if (user.length > 0) {
                userList.push(user);
            }
        }
        return userList;
    }    

    $scope.offlineChanged = function(license)
    {
        $scope.updateLicense(license);
    }

    $scope.$watch('selectAll', function () {
        var i;
        for (i = 0; i < $scope.licenses.length; i++) {
            $scope.licenses[i].checked = $scope.selectAll;
        }
    });
    $scope.$watch('canRefresh', function () {
        if ($scope.canRefresh) {
            $scope.refresh();
        }
    });

    $scope.showDialog = function (message, success) {
        var modalInstance = $modal.open({
            templateUrl: '/admin/partials/confirmationDialog.html',
            controller: 'ModalInstanceCtrl',
            resolve: {
                messagetext: function () {
                    return message;
                },
                showSuccess: function () {
                    return success;
                }
            },
            windowClass: 'wide-modal',
            backdrop: 'static'
        });
    };

    $scope.refresh();
})
.controller('LockedUsersController', function ($scope, $http,ErrorCode) {
    var inverse = '-';
    var empty = '';
    var separator = ',';
    var warningOnBlock = 'You are attempting to block more than 5 users. ' +                      
                      'Do you want to continue?';
    var warningOnRemove = 'You are attempting to remove more than 5 users. ' +
                      'You can only be able to unblock 5 users per day. ' +
                      'Do you want to continue?';
    var userUnblocked = 'The user has been unblocked.';
    var unBlockError = 'Error unblocking the user';
    var morethanFiveUserError = 'Cannot unblock more than 5 users per day.';
    //var canBlock = false;       
    //$scope.blockUserList = empty;
    $scope.canRefresh = false;
    $scope.orderbyItem = 'UserId';
    $scope.blockedUsers = {};

    $scope.refresh = function () {
        var futureResponse =
        $http(
                {
                    url: '/api/lockedUsers',
                    method: 'GET'
                }
            );

        futureResponse.success(function (data, status, headers, config) {
            $scope.blockedUsers = data;
            //$scope.blockUserList = empty;
        }

      );
    }

    //$scope.blockUser = function () {
    //    $scope.canRefresh = false;
    //    if ($scope.blockUserList != empty) {
    //        var blockUserArray = $scope.validateBlockUserList($scope.blockUserList);
    //        var blockUserCount = blockUserArray.length;
    //        var reponseCount = 0;
    //        if (blockUserCount > 0) {
    //            canBlock = true;
    //            if (blockUserCount > 5) {
    //                canBlock = false;
    //                if (window.confirm(warningOnBlock)) {
    //                    canBlock = true;
    //                }
    //            }
    //            if (canBlock) {
    //                angular.forEach(blockUserArray, function (blockUserId) {
    //                    var lockedUser = {
    //                        UserId: blockUserId,
    //                        Id: null
    //                    };
    //                    var futureResponse =
    //                        $http(
    //                        {
    //                            url: '/api/lockedUsers',
    //                            method: 'POST',
    //                            data: lockedUser
    //                        }
    //                    );
    //                    futureResponse.success(function (data, status, headers, config) {
    //                        reponseCount++;
    //                        if (reponseCount == blockUserCount) {
    //                            $scope.canRefresh = true;
    //                        }
    //                    });
    //                    futureResponse.error(function (data, status, headers, config) {
    //                        reponseCount++;
    //                        if (reponseCount == blockUserCount) {
    //                            $scope.canRefresh = true;
    //                        }
    //                        throw new Error('Error');
    //                    });

    //                });
    //            }
    //        }           
    //    }
    //}

    //$scope.validateBlockUserList = function (itemArray) {
    //    var list = itemArray.split(separator);
    //    var lcount = list.length;
    //    var user;
    //    var blockUserList = new Array();
    //    for (var i = 0; i < lcount; i++) {
    //        user = list[i].trim();
    //        if (user.length > 0) {
    //            blockUserList.push(user);
    //        }
    //    }
    //    return blockUserList;
    //}

    $scope.deleteBlockedUser = function () {
        
        var canDelete = true;
        var reponseCount = 0;
        var deleteCount = $scope.checkedCount();
        if (deleteCount >=5) {
            canDelete = false;
            if (window.confirm(warningOnRemove)) {
                canDelete = true;
            }
        }
        if (canDelete) {
            $scope.canRefresh = false;
            angular.forEach($scope.blockedUsers, function (user) {

                if (user.checked) {
                    var futureResponse =
                    $http(
                            {
                                url: '/api/lockedUsers/' + user.Id,
                                method: 'DELETE'
                            }
                        );
                    futureResponse.success(function (data, status, headers, config) {
                        reponseCount++;
                        if (reponseCount == deleteCount) {
                            $scope.canRefresh = true;
                            $scope.showDialog(userUnblocked, true);
                        }
                    });
                    futureResponse.error(function (data, status, headers, config) {
                        reponseCount++;
                        if (reponseCount == deleteCount) {
                            $scope.canRefresh = true;
                        }
                        if (status == ErrorCode.Forbidden) {
                            unBlockError = morethanFiveUserError;
                        } else if (status == ErrorCode.NotFound) {
                            unBlockError = 'User has been unblocked already';
                        } else if (status == ErrorCode.InternalServerError) {
                            unBlockError = data;
                        }
                        else {
                            unBlockError = status;
                        }                       
                        $scope.showDialog(unBlockError,false);                        
                    });
                }
            });
            $scope.refresh();
        }
    }

    $scope.$watch('canRefresh', function () {
        if ($scope.canRefresh) {
            $scope.refresh();
        }
    });

    $scope.checkedCount = function () {
        return $scope.blockedUsers.filter(function (user) {
            return user.checked;
        }).length;
    
    }

    $scope.sort = function (sortItem) {
        $scope.orderbyItem = sortItem;
    }

    $scope.inverseSort = function (sortItem) {
        $scope.orderbyItem = inverse.concat(sortItem);
    }

    $scope.showDialog = function (message,success) {
        var modalInstance = $modal.open({
            templateUrl: '/admin/partials/confirmationDialog.html',
            controller: 'ModalInstanceCtrl',
            resolve: {
                messagetext: function () {
                    return message;
                },
                showSuccess: function () {
                    return success;
                }
            },
            windowClass: 'wide-modal',
            backdrop: 'static'
        });
    }

    $scope.refresh();       
})
.controller('ModalInstanceCtrl', function ($scope, $modalInstance, messagetext, showSuccess) {
    $scope.message = messagetext;
    $scope.showSuccess = showSuccess;
    $scope.ok = function () {
        $modalInstance.dismiss('cancel');
    };    
})